import {Box, TextField, Button, Typography, Autocomplete} from '@mui/material'
import {Create} from '@refinedev/mui'
import {useForm} from '@refinedev/react-hook-form'
import {Controller} from 'react-hook-form'

import {MEDIA_TYPES} from '../../lib/constants'
import {ALLOWED_IMAGE_TYPES, ALLOWED_VIDEO_TYPES, useMediaFileUpload} from '../../components/media/MediaFormUtils'

type MediaFormValues = {
  filename: string
  media_type: number
  file?: File
}

export const MediaCreate = () => {
  const {
    saveButtonProps,
    refineCore: {formLoading, onFinish},
    register,
    control,
    formState: {errors},
    setValue,
    handleSubmit,
    watch,
    setError,
    clearErrors,
  } = useForm<MediaFormValues>({})

  const selectedMediaType = watch('media_type')

  const {selectedFile, previewUrl, handleFileChange, handleMediaTypeChange} = useMediaFileUpload({
    selectedMediaType,
    setError,
    clearErrors,
    setValue,
  })

  return (
    <Create
      isLoading={formLoading}
      saveButtonProps={{
        ...saveButtonProps,
        disabled: !!errors.file || !selectedFile,
        onClick: handleSubmit((data) => {
          const formData = new FormData()
          formData.append('filename', data.filename)
          formData.append('type', String(data.media_type))
          if (data.file) {
            formData.append('file', data.file)
          }
          onFinish(formData)
        }),
      }}
    >
      <Controller
        control={control}
        name="media_type"
        rules={{
          required: 'Это поле является обязательным',
        }}
        render={({field}) => (
          <Autocomplete
            options={MEDIA_TYPES}
            value={MEDIA_TYPES.find((option) => option.value === field.value) || null}
            onChange={(_, value) => {
              field.onChange(value?.value || null)
              handleMediaTypeChange(value?.value || null)
            }}
            getOptionLabel={(item) => {
              return item ? item.label : ''
            }}
            isOptionEqualToValue={(option, value) => {
              return option.value === value?.value
            }}
            renderInput={(params) => <TextField {...params} label="Тип" margin="normal" variant="outlined" error={!!errors.media_type} helperText={(errors as any)?.media_type?.message} required />}
          />
        )}
      />

      <TextField
        {...register('filename', {
          required: 'Это поле является обязательным',
        })}
        error={!!(errors as any)?.filename}
        helperText={(errors as any)?.filename?.message}
        margin="normal"
        fullWidth
        InputLabelProps={{shrink: true}}
        type="text"
        label="Название"
        name="filename"
      />

      <Box component="form" sx={{display: 'flex', flexDirection: 'column'}} autoComplete="off">
        <Box display="flex" flexDirection="column-reverse" alignItems="center" gap={6}>
          <Box display="flex" flexDirection="column" alignItems="center" gap={2}>
            <Button variant="contained" component="label" disabled={!selectedMediaType}>
              {selectedFile ? 'Изменить файл' : 'Загрузить файл'}
              <input type="file" hidden onChange={handleFileChange} accept={selectedMediaType === 1 ? ALLOWED_IMAGE_TYPES.join(',') : ALLOWED_VIDEO_TYPES.join(',')} />
            </Button>

            {selectedFile && (
              <Typography variant="body2" color="text.secondary">
                {selectedFile.name}
              </Typography>
            )}

            {errors.file && (
              <Typography variant="caption" color="error">
                {(errors as any)?.file?.message}
              </Typography>
            )}
          </Box>

          {previewUrl && (
            <Box mt={2} display="flex" justifyContent="center">
              <img src={previewUrl} alt="Preview" style={{maxWidth: '200px', borderRadius: 8}} />
            </Box>
          )}
        </Box>
      </Box>
    </Create>
  )
}
